@@ -1,4 +1,4 @@
-from redis.client import Redis
+from redis.client import Redis, StrictRedis
 from redis.connection import (
     ConnectionPool,
     Connection,
@@ -16,11 +16,12 @@ from redis.exceptions import (
     )
 
 
-__version__ = '2.4.8'
+__version__ = '2.4.10'
 VERSION = tuple(map(int, __version__.split('.')))
 
 __all__ = [
-    'Redis', 'ConnectionPool', 'Connection', 'UnixDomainSocketConnection',
+    'Redis', 'StrictRedis', 'ConnectionPool',
+    'Connection', 'UnixDomainSocketConnection',
     'RedisError', 'ConnectionError', 'ResponseError', 'AuthenticationError',
     'InvalidResponse', 'DataError', 'PubSubError', 'WatchError',
     ]
