@@ -1,3 +1,4 @@
+from __future__ import with_statement
 import datetime
 import time
 import warnings
@@ -43,9 +44,32 @@ def dict_merge(*dicts):
     [merged.update(d) for d in dicts]
     return merged
 
+def parse_debug_object(response):
+    "Parse the results of Redis's DEBUG OBJECT command into a Python dict"
+    # The 'type' of the object is the first item in the response, but isn't
+    # prefixed with a name
+    response = 'type:' + response
+    response = dict([kv.split(':') for kv in response.split()])
+
+    # parse some expected int values from the string response
+    # note: this cmd isn't spec'd so these may not appear in all redis versions
+    int_fields = ('refcount', 'serializedlength', 'lru', 'lru_seconds_idle')
+    for field in int_fields:
+        if field in response:
+            response[field] = int(response[field])
+
+    return response
+
+def parse_object(response, infotype):
+    "Parse the results of an OBJECT command"
+    if infotype in ('idletime', 'refcount'):
+        return int(response)
+    return response
+
 def parse_info(response):
     "Parse the result of Redis's INFO command into a Python dict"
     info = {}
+
     def get_value(value):
         if ',' not in value:
             return value
@@ -101,7 +125,7 @@ def parse_config(response, **options):
         return response and pairs_to_dict(response) or {}
     return response == 'OK'
 
-class Redis(object):
+class StrictRedis(object):
     """
     Implementation of the Redis protocol.
 
@@ -114,13 +138,13 @@ class Redis(object):
     RESPONSE_CALLBACKS = dict_merge(
         string_keys_to_dict(
             'AUTH DEL EXISTS EXPIRE EXPIREAT HDEL HEXISTS HMSET MOVE MSETNX '
-            'PERSIST RENAMENX SADD SISMEMBER SMOVE SETEX SETNX SREM ZADD ZREM',
+            'PERSIST RENAMENX SISMEMBER SMOVE SETEX SETNX SREM ZREM',
             bool
             ),
         string_keys_to_dict(
-            'DECRBY GETBIT HLEN INCRBY LINSERT LLEN LPUSHX RPUSHX SCARD '
-            'SDIFFSTORE SETBIT SETRANGE SINTERSTORE STRLEN SUNIONSTORE ZCARD '
-            'ZREMRANGEBYRANK ZREMRANGEBYSCORE',
+            'DECRBY GETBIT HLEN INCRBY LINSERT LLEN LPUSHX RPUSHX SADD SCARD '
+            'SDIFFSTORE SETBIT SETRANGE SINTERSTORE STRLEN SUNIONSTORE ZADD '
+            'ZCARD ZREMRANGEBYRANK ZREMRANGEBYSCORE',
             int
             ),
         string_keys_to_dict(
@@ -148,12 +172,13 @@ class Redis(object):
             'BGSAVE': lambda r: r == 'Background saving started',
             'BRPOPLPUSH': lambda r: r and r or None,
             'CONFIG': parse_config,
+            'DEBUG': parse_debug_object,
             'HGETALL': lambda r: r and pairs_to_dict(r) or {},
             'INFO': parse_info,
             'LASTSAVE': timestamp_to_datetime,
+            'OBJECT': parse_object,
             'PING': lambda r: r == 'PONG',
             'RANDOMKEY': lambda r: r and r or None,
-            'TTL': lambda r: r != -1 and r or None,
         }
         )
 
@@ -197,7 +222,7 @@ class Redis(object):
         atomic, pipelines are useful for reducing the back-and-forth overhead
         between the client and server.
         """
-        return Pipeline(
+        return StrictPipeline(
             self.connection_pool,
             self.response_callbacks,
             transaction,
@@ -288,11 +313,19 @@ class Redis(object):
         "Returns the number of keys in the current database"
         return self.execute_command('DBSIZE')
 
+    def debug_object(self, key):
+        "Returns version specific metainformation about a give key"
+        return self.execute_command('DEBUG', 'OBJECT', key)
+
     def delete(self, *names):
         "Delete one or more keys specified by ``names``"
         return self.execute_command('DEL', *names)
     __delitem__ = delete
 
+    def echo(self, value):
+        "Echo the string back from the server"
+        return self.execute_command('ECHO', value)
+
     def flushall(self):
         "Delete all keys in all databases on the current host"
         return self.execute_command('FLUSHALL')
@@ -312,6 +345,10 @@ class Redis(object):
         """
         return self.execute_command('LASTSAVE')
 
+    def object(self, infotype, key):
+        "Return the encoding, idletime, or refcount about the key"
+        return self.execute_command('OBJECT', infotype, key, infotype=infotype)
+
     def ping(self):
         "Ping the Redis server"
         return self.execute_command('PING')
@@ -473,7 +510,7 @@ class Redis(object):
         value = value and 1 or 0
         return self.execute_command('SETBIT', name, offset, value)
 
-    def setex(self, name, value, time):
+    def setex(self, name, time, value):
         """
         Set the value of key ``name`` to ``value``
         that expires in ``time`` seconds
@@ -627,13 +664,17 @@ class Redis(object):
         """
         return self.execute_command('LRANGE', name, start, end)
 
-    def lrem(self, name, value, num=0):
+    def lrem(self, name, count, value):
         """
-        Remove the first ``num`` occurrences of ``value`` from list ``name``
+        Remove the first ``count`` occurrences of elements equal to ``value``
+        from the list stored at ``name``.
 
-        If ``num`` is 0, then all occurrences will be removed
+        The count argument influences the operation in the following ways:
+            count > 0: Remove elements equal to value moving from head to tail.
+            count < 0: Remove elements equal to value moving from tail to head.
+            count = 0: Remove all elements equal to value.
         """
-        return self.execute_command('LREM', name, num, value)
+        return self.execute_command('LREM', name, count, value)
 
     def lset(self, name, index, value):
         "Set ``position`` of list ``name`` to ``value``"
@@ -797,27 +838,27 @@ class Redis(object):
 
 
     #### SORTED SET COMMANDS ####
-    def zadd(self, name, value=None, score=None, **pairs):
+    def zadd(self, name, *args, **kwargs):
         """
-        For each kwarg in ``pairs``, add that item and it's score to the
-        sorted set ``name``.
+        Set any number of score, element-name pairs to the key ``name``. Pairs
+        can be specified in two ways:
 
-        The ``value`` and ``score`` arguments are deprecated.
+        As *args, in the form of: score1, name1, score2, name2, ...
+        or as **kwargs, in the form of: name1=score1, name2=score2, ...
+
+        The following example would add four values to the 'my-key' key:
+        redis.zadd('my-key', 1.1, 'name1', 2.2, 'name2', name3=3.3, name4=4.4)
         """
-        all_pairs = []
-        if value is not None or score is not None:
-            if value is None or score is None:
-                raise RedisError("Both 'value' and 'score' must be specified " \
-                                 "to ZADD")
-            warnings.warn(DeprecationWarning(
-                "Passing 'value' and 'score' has been deprecated. " \
-                "Please pass via kwargs instead."))
-            all_pairs.append(score)
-            all_pairs.append(value)
-        for pair in pairs.iteritems():
-            all_pairs.append(pair[1])
-            all_pairs.append(pair[0])
-        return self.execute_command('ZADD', name, *all_pairs)
+        pieces = []
+        if args:
+            if len(args) % 2 != 0:
+                raise RedisError("ZADD requires an equal number of "
+                                 "values and scores")
+            pieces.extend(args)
+        for pair in kwargs.iteritems():
+            pieces.append(pair[1])
+            pieces.append(pair[0])
+        return self.execute_command('ZADD', name, *pieces)
 
     def zcard(self, name):
         "Return the number of elements in the sorted set ``name``"
@@ -1062,6 +1103,86 @@ class Redis(object):
         return self.execute_command('PUBLISH', channel, message)
 
 
+class Redis(StrictRedis):
+    """
+    Provides backwards compatibility with older versions of redis-py that
+    changed arguments to some commands to be more Pythonic, sane, or by
+    accident.
+    """
+
+    # Overridden callbacks
+    RESPONSE_CALLBACKS = dict_merge(
+        StrictRedis.RESPONSE_CALLBACKS,
+        {
+            'TTL': lambda r: r != -1 and r or None,
+        }
+    )
+
+    def pipeline(self, transaction=True, shard_hint=None):
+        """
+        Return a new pipeline object that can queue multiple commands for
+        later execution. ``transaction`` indicates whether all commands
+        should be executed atomically. Apart from making a group of operations
+        atomic, pipelines are useful for reducing the back-and-forth overhead
+        between the client and server.
+        """
+        return Pipeline(
+            self.connection_pool,
+            self.response_callbacks,
+            transaction,
+            shard_hint)
+
+    def setex(self, name, value, time):
+        """
+        Set the value of key ``name`` to ``value``
+        that expires in ``time`` seconds
+        """
+        return self.execute_command('SETEX', name, time, value)
+
+    def lrem(self, name, value, num=0):
+        """
+        Remove the first ``num`` occurrences of elements equal to ``value``
+        from the list stored at ``name``.
+
+        The ``num`` argument influences the operation in the following ways:
+            num > 0: Remove elements equal to value moving from head to tail.
+            num < 0: Remove elements equal to value moving from tail to head.
+            num = 0: Remove all elements equal to value.
+        """
+        return self.execute_command('LREM', name, num, value)
+
+    def zadd(self, name, *args, **kwargs):
+        """
+        NOTE: The order of arguments differs from that of the official ZADD
+        command. For backwards compatability, this method accepts arguments
+        in the form of name1, score1, name2, score2, while the official Redis
+        documents expects score1, name1, score2, name2.
+
+        If you're looking to use the standard syntax, consider using the
+        StrictRedis class. See the API Reference section of the docs for more
+        information.
+
+        Set any number of element-name, score pairs to the key ``name``. Pairs
+        can be specified in two ways:
+
+        As *args, in the form of: name1, score1, name2, score2, ...
+        or as **kwargs, in the form of: name1=score1, name2=score2, ...
+
+        The following example would add four values to the 'my-key' key:
+        redis.zadd('my-key', 'name1', 1.1, 'name2', 2.2, name3=3.3, name4=4.4)
+        """
+        pieces = []
+        if args:
+            if len(args) % 2 != 0:
+                raise RedisError("ZADD requires an equal number of "
+                                 "values and scores")
+            pieces.extend(reversed(args))
+        for pair in kwargs.iteritems():
+            pieces.append(pair[1])
+            pieces.append(pair[0])
+        return self.execute_command('ZADD', name, *pieces)
+
+
 class PubSub(object):
     """
     PubSub provides publish, subscribe and listen support to Redis channels.
@@ -1081,6 +1202,22 @@ class PubSub(object):
             ('subscribe', 'psubscribe', 'unsubscribe', 'punsubscribe')
             )
 
+    def __del__(self):
+        try:
+            # if this object went out of scope prior to shutting down
+            # subscriptions, close the connection manually before
+            # returning it to the connection pool
+            if self.connection and (self.channels or self.patterns):
+                self.connection.disconnect()
+            self.reset()
+        except:
+            pass
+
+    def reset(self):
+        if self.connection:
+            self.connection_pool.release(self.connection)
+            self.connection = None
+
     def execute_command(self, *args, **kwargs):
         "Execute a publish/subscribe command"
         if self.connection is None:
@@ -1094,6 +1231,9 @@ class PubSub(object):
             return self.parse_response()
         except ConnectionError:
             connection.disconnect()
+            # Connect manually here. If the Redis server is down, this will
+            # fail and raise a ConnectionError as desired.
+            connection.connect()
             # resubscribe to all channels and patterns before
             # resending the current command
             for channel in self.channels:
@@ -1111,8 +1251,7 @@ class PubSub(object):
             # if we've just unsubscribed from the remaining channels,
             # release the connection back to the pool
             if not self.subscription_count:
-                self.connection_pool.release(self.connection)
-                self.connection = None
+                self.reset()
         return response
 
     def psubscribe(self, patterns):
@@ -1180,7 +1319,7 @@ class PubSub(object):
             yield msg
 
 
-class Pipeline(Redis):
+class BasePipeline(object):
     """
     Pipelines provide a way to transmit multiple commands to the Redis server
     in one transmission.  This is convenient for batch processing, such as
@@ -1218,6 +1357,12 @@ class Pipeline(Redis):
     def __exit__(self, exc_type, exc_value, traceback):
         self.reset()
 
+    def __del__(self):
+        try:
+            self.reset()
+        except:
+            pass
+
     def reset(self):
         self.command_stack = []
         # make sure to reset the connection state in the event that we were
@@ -1340,8 +1485,8 @@ class Pipeline(Redis):
                 for args, options in commands]
 
     def parse_response(self, connection, command_name, **options):
-        result = super(Pipeline, self).parse_response(
-            connection, command_name, **options)
+        result = StrictRedis.parse_response(
+            self, connection, command_name, **options)
         if command_name in self.UNWATCH_COMMANDS:
             self.watching = False
         elif command_name == 'WATCH':
@@ -1352,7 +1497,7 @@ class Pipeline(Redis):
         "Execute all the commands in the current pipeline"
         stack = self.command_stack
         if self.transaction or self.explicit_transaction:
-            stack = [(('MULTI' ,), {})] + stack + [(('EXEC', ), {})]
+            stack = [(('MULTI', ), {})] + stack + [(('EXEC', ), {})]
             execute = self._execute_transaction
         else:
             execute = self._execute_pipeline
@@ -1397,6 +1542,14 @@ class Pipeline(Redis):
         return self.watching and self.execute_command('UNWATCH') or True
 
 
+class StrictPipeline(BasePipeline, StrictRedis):
+    "Pipeline for the StrictRedis class"
+    pass
+
+class Pipeline(BasePipeline, Redis):
+    "Pipeline for the Redis class"
+    pass
+
 class LockError(RedisError):
     "Errors thrown from the Lock"
     pass
@@ -1425,8 +1578,8 @@ class Lock(object):
         holding the lock.
 
         Note: If using ``timeout``, you should make sure all the hosts
-        that are running clients are within the same timezone and are using
-        a network time service like ntp.
+        that are running clients have their time synchronized with a network time
+        service like ntp.
         """
         self.redis = redis
         self.name = name
