@@ -1,12 +1,25 @@
-import errno
 import socket
 from itertools import chain, imap
 from redis.exceptions import ConnectionError, ResponseError, InvalidResponse
 
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
+
 class PythonParser(object):
+    "Plain Python parsing class"
+    MAX_READ_LENGTH = 1000000
+
     def __init__(self):
         self._fp = None
 
+    def __del__(self):
+        try:
+            self.on_disconnect()
+        except:
+            pass
+
     def on_connect(self, connection):
         "Called when the socket connects"
         self._fp = connection._sock.makefile('r')
@@ -24,7 +37,25 @@ class PythonParser(object):
         """
         try:
             if length is not None:
-                return self._fp.read(length+2)[:-2]
+                bytes_left = length + 2 # read the line ending
+                if length > self.MAX_READ_LENGTH:
+                    # apparently reading more than 1MB or so from a windows
+                    # socket can cause MemoryErrors. See:
+                    # https://github.com/andymccurdy/redis-py/issues/205
+                    # read smaller chunks at a time to work around this
+                    try:
+                        buf = StringIO()
+                        while bytes_left > 0:
+                            read_len = min(bytes_left, self.MAX_READ_LENGTH)
+                            buf.write(self._fp.read(read_len))
+                            bytes_left -= read_len
+                        buf.seek(0)
+                        return buf.read(length)
+                    finally:
+                        buf.close()
+                return self._fp.read(bytes_left)[:-2]
+
+            # no length, read a full line
             return self._fp.readline()[:-2]
         except (socket.error, socket.timeout), e:
             raise ConnectionError("Error while reading from socket: %s" % \
@@ -68,6 +99,13 @@ class PythonParser(object):
         raise InvalidResponse("Protocol Error")
 
 class HiredisParser(object):
+    "Parser class for connections using Hiredis"
+    def __del__(self):
+        try:
+            self.on_disconnect()
+        except:
+            pass
+
     def on_connect(self, connection):
         self._sock = connection._sock
         self._reader = hiredis.Reader(
@@ -79,6 +117,8 @@ class HiredisParser(object):
         self._reader = None
 
     def read_response(self):
+        if not self._reader:
+            raise ConnectionError("Socket closed on remote end")
         response = self._reader.gets()
         while response is False:
             try:
@@ -117,6 +157,12 @@ class Connection(object):
         self._sock = None
         self._parser = parser_class()
 
+    def __del__(self):
+        try:
+            self.disconnect()
+        except:
+            pass
+
     def connect(self):
         "Connects to the Redis server if not already connected"
         if self._sock:
@@ -146,7 +192,6 @@ class Connection(object):
             return "Error %s connecting %s:%s. %s." % \
                 (exception.args[0], self.host, self.port, exception.args[1])
 
-
     def on_connect(self):
         "Initialize the connection, authenticate and select a database"
         self._parser.on_connect(self)
@@ -174,32 +219,23 @@ class Connection(object):
             pass
         self._sock = None
 
-    def _send(self, command):
-        "Send the command to the socket"
+    def send_packed_command(self, command):
+        "Send an already packed command to the Redis server"
         if not self._sock:
             self.connect()
         try:
             self._sock.sendall(command)
         except socket.error, e:
-            if e.args[0] == errno.EPIPE:
-                self.disconnect()
+            self.disconnect()
             if len(e.args) == 1:
                 _errno, errmsg = 'UNKNOWN', e.args[0]
             else:
                 _errno, errmsg = e.args
             raise ConnectionError("Error %s while writing to socket. %s." % \
                 (_errno, errmsg))
-
-    def send_packed_command(self, command):
-        "Send an already packed command to the Redis server"
-        try:
-            self._send(command)
-        except ConnectionError:
-            # retry the command once in case the socket connection simply
-            # timed out
+        except:
             self.disconnect()
-            # if this _send() call fails, then the error will be raised
-            self._send(command)
+            raise
 
     def send_command(self, *args):
         "Pack and send a command to the Redis server"
@@ -207,7 +243,11 @@ class Connection(object):
 
     def read_response(self):
         "Read the response from a previously sent command"
-        response = self._parser.read_response()
+        try:
+            response = self._parser.read_response()
+        except:
+            self.disconnect()
+            raise
         if response.__class__ == ResponseError:
             raise response
         return response
